@@ -8,9 +8,9 @@ require_once 'Horde/Serialize.php';
  * query information about individual mailboxes.
  * In IMP, folders = IMAP mailboxes so the two terms are used interchangably.
  *
- * $Horde: imp/lib/IMAP/Tree.php,v 1.25.2.64 2008/12/10 05:13:08 slusarz Exp $
+ * $Horde: imp/lib/IMAP/Tree.php,v 1.25.2.67 2009/01/06 15:24:05 jan Exp $
  *
- * Copyright 2000-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2000-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -185,6 +185,13 @@ class IMP_Tree {
      */
     var $_eltdiff = null;
 
+    /**
+     * If set, track element changes.
+     *
+     * @var boolean
+     */
+    var $_trackdiff = true;
+
     /**
      * See $open parameter in build().
      *
@@ -259,10 +266,11 @@ class IMP_Tree {
          * user MUST call reset() before cycling through the tree.
          * Don't store $_subscribed - this information is stored in the
          * elements.
-         * Reset the $_changed flag. */
+         * Reset the $_changed and $_trackdiff flags. */
         $this->_currkey = $this->_currparent = $this->_eltdiff = $this->_expanded = $this->_imap_sort = $this->_poll = $this->_subscribed = null;
         $this->_currstack = array();
         $this->_changed = false;
+        $this->_trackdiff = true;
 
         if (!isset($_SESSION['imp']['cache']['imp_tree'])) {
             $_SESSION['imp']['cache']['imp_tree'] = array(
@@ -782,13 +790,17 @@ class IMP_Tree {
             // This is a case where it is possible that the parent element has
             // changed (it now has children) but we can't catch it via the
             // bitflag (since hasChildren() is dynamically determined).
-            if (isset($this->_eltdiff[$elt['p']])) {
-                $this->_eltdiff[$elt['p']]['changed'] = true;
+            if ($this->_trackdiff && !is_null($this->_eltdiff)) {
+                $this->_eltdiff['c'][$elt['p']] = 1;
             }
         }
         $this->_parent[$elt['p']][] = $elt['v'];
         $this->_tree[$elt['v']] = $elt;
 
+        if ($this->_trackdiff && !is_null($this->_eltdiff)) {
+            $this->_eltdiff['a'][$elt['v']] = 1;
+        }
+
         /* Make sure we are sorted correctly. */
         if (count($this->_parent[$elt['p']]) > 1) {
             $this->_setNeedSort($this->_tree[$elt['p']], true);
@@ -878,6 +890,10 @@ class IMP_Tree {
         $key = array_search($id, $this->_parent[$parent]);
         unset($this->_parent[$parent][$key]);
 
+        if (!is_null($this->_eltdiff)) {
+            $this->_eltdiff['d'][$id] = 1;
+        }
+
         if (empty($this->_parent[$parent])) {
             /* This folder is now completely empty (no children).  If the
              * folder is a container only, we should delete the folder from
@@ -894,8 +910,8 @@ class IMP_Tree {
                      * element has changed (it no longer has children) but
                      * we can't catch it via the bitflag (since hasChildren()
                      * is dynamically determined). */
-                    if (isset($this->_eltdiff[$parent])) {
-                        $this->_eltdiff[$parent]['changed'] = true;
+                    if (!is_null($this->_eltdiff)) {
+                        $this->_eltdiff['c'][$parent] = 1;
                     }
                 }
             }
@@ -1374,7 +1390,9 @@ class IMP_Tree {
          * to add all unsubscribed elements that live in currently
          * discovered items. */
         $this->_unsubview = true;
+        $this->_trackdiff = false;
         $this->insert($this->_getList(true));
+        $this->_trackdiff = true;
     }
 
     /**
@@ -1505,7 +1523,7 @@ class IMP_Tree {
      */
     function eltDiffStart()
     {
-        $this->_eltdiff = $this->_tree;
+        $this->_eltdiff = array('a' => array(), 'c' => array(), 'd' => array());
     }
 
     /**
@@ -1514,38 +1532,29 @@ class IMP_Tree {
      *
      * @since IMP 4.1
      *
-     * @return array  An array with the following keys:
+     * @return array  Returns false if no changes have occurred, or an array
+     *                with the following keys:
      * <pre>
      * 'a' => A list of elements that have been added.
      * 'c' => A list of elements that have been changed.
      * 'd' => A list of elements that have been deleted.
      * </pre>
-     *                Returns false if no changes have occurred.
      */
     function eltDiff()
     {
-        if (!$this->_changed || !$this->_eltdiff) {
+        if (is_null($this->_eltdiff) || !$this->_changed) {
             return false;
         }
 
-        $added = $changed = $deleted = array();
+        $ret = array(
+            'a' => array_keys($this->_eltdiff['a']),
+            'c' => array_keys($this->_eltdiff['c']),
+            'd' => array_keys($this->_eltdiff['d'])
+        );
 
-        /* Determine the deleted items. */
-        $deleted = array_values(array_diff(array_keys($this->_eltdiff), array_keys($this->_tree)));
+        $this->_eltdiff = null;
 
-        foreach ($this->_tree as $key => $val) {
-            if (!isset($this->_eltdiff[$key])) {
-                $added[] = $key;
-            } elseif ($val != $this->_eltdiff[$key]) {
-                $changed[] = $key;
-            }
-        }
-
-        if (empty($added) && empty($changed) && empty($deleted)) {
-            return false;
-        } else {
-            return array('a' => $added, 'c' => $changed, 'd' => $deleted);
-        }
+        return $ret;
     }
 
     /**
