@@ -4,16 +4,15 @@
  * and provides an option to import the data into a calendar source,
  * if one is available.
  *
- * $Horde: imp/lib/MIME/Viewer/itip.php,v 1.37.2.27 2007/03/06 15:25:33 jan Exp $
+ * $Horde: imp/lib/MIME/Viewer/itip.php,v 1.37.2.44 2009/01/07 18:53:20 jan Exp $
  *
- * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2009 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL). If you
- * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * See the enclosed file COPYING for license information (GPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Mike Cochrane <mike@graftonhall.co.nz>
- * @since   IMP 4.0
  * @package Horde_MIME_Viewer
  */
 class IMP_MIME_Viewer_itip extends MIME_Viewer {
@@ -67,16 +66,15 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
 
         // Parse the iCal file.
         $vCal = new Horde_iCalendar();
-        if (!$vCal->parsevCalendar($data)) {
+        if (!$vCal->parsevCalendar($data, 'VCALENDAR', $this->mime_part->getCharset())) {
             return '<h1>' . _("The calendar data is invalid") . '</h1>' .
                 '<pre>' . htmlspecialchars($data) . '</pre>';
         }
 
-        // Check if we got vcard data with the wrong vcalendar mime
-        // type.
+        // Check if we got vcard data with the wrong vcalendar mime type.
         $c = $vCal->getComponentClasses();
         if (count($c) == 1 && !empty($c['horde_icalendar_vcard'])) {
-            $vcard_renderer = &Mime_Viewer::factory($this->mime_part, 'text/x-vcard');
+            $vcard_renderer = &MIME_Viewer::factory($this->mime_part, 'text/x-vcard');
             return $vcard_renderer->render($params);
         }
 
@@ -100,7 +98,7 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                     $guid = $components[$key]->getAttribute('UID');
                     $event = $registry->call('calendar/delete', array('guid' => $guid));
                     if (is_a($event, 'PEAR_Error')) {
-                        $this->_msgs[$key][] = array('error', sprintf(_("There was an error deleting the event: %s."), $event->getMessage()));
+                        $this->_msgs[$key][] = array('error', _("There was an error deleting the event:") . ' ' . $event->getMessage());
                     } else {
                         $this->_msgs[$key][] = array('success', _("Event successfully deleted."));
                     }
@@ -112,9 +110,9 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
             case 'update':
                 // vEvent reply.
                 if ($registry->hasMethod('calendar/updateAttendee')) {
-                    $event = $registry->call('calendar/updateAttendee', array('response' => $components[$key]));
+                    $event = $registry->call('calendar/updateAttendee', array('response' => $components[$key], 'sender' => $params[0]->getFromAddress()));
                     if (is_a($event, 'PEAR_Error')) {
-                        $this->_msgs[$key][] = array('error', sprintf(_("There was an error updating the event: %s."), $event->getMessage()));
+                        $this->_msgs[$key][] = array('error', _("There was an error updating the event:") . ' ' . $event->getMessage());
                     } else {
                         $this->_msgs[$key][] = array('success', _("Respondent Status Updated."));
                     }
@@ -133,17 +131,38 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                 // vJournal publish.
                 switch ($components[$key]->getType()) {
                 case 'vEvent':
-                    // Import into Kronolith.
-                    if ($registry->hasMethod('calendar/import')) {
-                        $guid = $registry->call('calendar/import', array('content' => $components[$key], 'contentType' => 'text/calendar'));
-                        if (is_a($guid, 'PEAR_Error')) {
-                            $this->_msgs[$key][] = array('error', sprintf(_("There was an error importing the event: %s."), $guid->getMessage()));
-                        } else {
-                            $url = Horde::url($registry->link('calendar/show', array('uid' => $guid)));
-                            $this->_msgs[$key][] = array('success', _("The event was added to your calendar.") .
-                                                         '&nbsp;' . Horde::link($url, _("View event"), null, '_blank') . Horde::img('mime/icalendar.png', _("View event"), null, $registry->getImageDir('horde')) . '</a>');
+                    $handled = false;
+                    $guid = $components[$key]->getAttribute('UID');
+                    // Check if this is an update.
+                    if ($registry->hasMethod('calendar/export') &&
+                        !is_a($registry->call('calendar/export', array($guid, 'text/calendar')), 'PEAR_Error')) {
+                        // Update in Kronolith.
+                        if ($registry->hasMethod('calendar/replace')) {
+                            $handled = true;
+                            $result = $registry->call('calendar/replace', array('uid' => $guid, 'content' => $components[$key], 'contentType' => $this->mime_part->getType()));
+                            if (is_a($result, 'PEAR_Error')) {
+                                $this->_msgs[$key][] = array('error', _("There was an error updating the event:") . ' ' . $result->getMessage());
+                            } else {
+                                $url = Horde::url($registry->link('calendar/show', array('uid' => $guid)));
+                                $this->_msgs[$key][] = array('success', _("The event was updated in your calendar.") .
+                                                             '&nbsp;' . Horde::link($url, _("View event"), null, '_blank') . Horde::img('mime/icalendar.png', _("View event"), null, $registry->getImageDir('horde')) . '</a>');
+                            }
                         }
                     } else {
+                        // Import into Kronolith.
+                        if ($registry->hasMethod('calendar/import')) {
+                            $handled = true;
+                            $guid = $registry->call('calendar/import', array('content' => $components[$key], 'contentType' => $this->mime_part->getType()));
+                            if (is_a($guid, 'PEAR_Error')) {
+                                $this->_msgs[$key][] = array('error', _("There was an error importing the event:") . ' ' . $guid->getMessage());
+                            } else {
+                                $url = Horde::url($registry->link('calendar/show', array('uid' => $guid)));
+                                $this->_msgs[$key][] = array('success', _("The event was added to your calendar.") .
+                                                             '&nbsp;' . Horde::link($url, _("View event"), null, '_blank') . Horde::img('mime/icalendar.png', _("View event"), null, $registry->getImageDir('horde')) . '</a>');
+                            }
+                        }
+                    }
+                    if (!$handled) {
                         $this->_msgs[$key][] = array('warning', _("This action is not supported."));
                     }
                     break;
@@ -153,7 +172,7 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                     if ($registry->hasMethod('calendar/import_vfreebusy')) {
                         $res = $registry->call('calendar/import_vfreebusy', array($components[$key]));
                         if (is_a($res, 'PEAR_Error')) {
-                            $this->_msgs[$key][] = array('error', sprintf(_("There was an error importing user's free/busy information: %s."), $res->getMessage()));
+                            $this->_msgs[$key][] = array('error', _("There was an error importing user's free/busy information:") . ' ' . $res->getMessage());
                         } else {
                             $this->_msgs[$key][] = array('success', _("The user's free/busy information was sucessfully stored."));
                         }
@@ -165,9 +184,9 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                 case 'vTodo':
                     // Import into Nag.
                     if ($registry->hasMethod('tasks/import')) {
-                        $guid = $registry->call('tasks/import', array($components[$key], 'text/x-vtodo'));
+                        $guid = $registry->call('tasks/import', array($components[$key], $this->mime_part->getType()));
                         if (is_a($guid, 'PEAR_Error')) {
-                            $this->_msgs[$key][] = array('error', sprintf(_("There was an error importing the task: %s."), $guid->getMessage()));
+                            $this->_msgs[$key][] = array('error', _("There was an error importing the task:") . ' ' . $guid->getMessage());
                         } else {
                             $url = Horde::url($registry->link('tasks/show', array('uid' => $guid)));
                             $this->_msgs[$key][] = array('success', _("The task has been added to your tasklist.") .
@@ -196,17 +215,21 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                     $components[$key]->getType() == 'vEvent') {
                     $vEvent = $components[$key];
 
+                    // Get the organizer details.
+                    $organizer = $vEvent->getAttribute('ORGANIZER');
+                    if (is_a($organizer, 'PEAR_Error')) {
+                        break;
+                    }
+                    $organizer = parse_url($organizer);
+                    $organizerEmail = $organizer['path'];
+                    $organizer = $vEvent->getAttribute('ORGANIZER', true);
+                    $organizerName = isset($organizer['cn']) ? $organizer['cn'] : '';
+
                     require_once 'Horde/Identity.php';
                     require_once 'Horde/MIME.php';
                     require_once 'Horde/MIME/Headers.php';
                     require_once 'Horde/MIME/Part.php';
 
-                    // Get the organizer details.
-                    $organizer = parse_url($vEvent->getAttribute('ORGANIZER'));
-                    $organizerEmail = $organizer['path'];
-                    $organizer = $vEvent->getAttribute('ORGANIZER', true);
-                    $organizerName = isset($organizer['cn']) ? $organizer['cn'] : '';
-
                     // Build the reply.
                     $vCal = new Horde_iCalendar();
                     $vCal->setAttribute('PRODID', '-//The Horde Project//' . HORDE_AGENT_HEADER . '//EN');
@@ -232,7 +255,6 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                     if (!is_a($vEvent->getAttribute('SEQUENCE'), 'PEAR_error')) {
                         $vEvent_reply->setAttribute('SEQUENCE', $vEvent->getAttribute('SEQUENCE'));
                     }
-                    $organizer = $vEvent->getAttribute('ORGANIZER', true);
                     $vEvent_reply->setAttribute('ORGANIZER', $vEvent->getAttribute('ORGANIZER'), array_pop($organizer));
 
                     // Find out who we are and update status.
@@ -248,26 +270,29 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                             break;
                         }
                     }
-                    $email = $identity->getFromAddress();
+                    $name = $email = $identity->getFromAddress();
+                    $params = array();
                     $cn = $identity->getValue('fullname');
-                    $params = array('CN' => $cn);
+                    if (!empty($cn)) {
+                        $name = $params['CN'] = $cn;
+                    }
 
                     switch ($action) {
                     case 'accept':
                     case 'accept-import':
-                        $message = sprintf(_("%s has accepted."), $cn);
+                        $message = sprintf(_("%s has accepted."), $name);
                         $subject = _("Accepted: ") . $vEvent->getAttribute('SUMMARY');
                         $params['PARTSTAT'] = 'ACCEPTED';
                         break;
 
                     case 'deny':
-                        $message = sprintf(_("%s has declined."), $cn);
+                        $message = sprintf(_("%s has declined."), $name);
                         $subject = _("Declined: ") . $vEvent->getAttribute('SUMMARY');
                         $params['PARTSTAT'] = 'DECLINED';
                         break;
 
                     case 'tentative':
-                        $message = sprintf(_("%s has tentatively accepted."), $cn);
+                        $message = sprintf(_("%s has tentatively accepted."), $name);
                         $subject = _("Tentative: ") . $vEvent->getAttribute('SUMMARY');
                         $params['PARTSTAT'] = 'TENTATIVE';
                         break;
@@ -307,7 +332,10 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                     $msg_headers->addMIMEHeaders($mime);
 
                     // Send the reply.
-                    $status = $mime->send($organizerEmail, $msg_headers);
+                    $mail_driver = $this->_getMailDriver();
+                    $status = $mime->send($organizerEmail, $msg_headers,
+                                          $mail_driver['driver'],
+                                          $mail_driver['params']);
                     if (is_a($status, 'PEAR_Error')) {
                         $this->_msgs[$key][] = array('error', sprintf(_("Error sending reply: %s."), $status->getMessage()));
                     } else {
@@ -332,6 +360,17 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                 if (isset($components[$key]) &&
                     $components[$key]->getType() == 'vFreebusy') {
                     $vFb = $components[$key];
+
+                    // Get the organizer details.
+                    $organizer = $vFb->getAttribute('ORGANIZER');
+                    if (is_a($organizer, 'PEAR_Error')) {
+                        break;
+                    }
+                    $organizer = parse_url($organizer);
+                    $organizerEmail = $organizer['path'];
+                    $organizer = $vFb->getAttribute('ORGANIZER', true);
+                    $organizerName = isset($organizer['cn']) ? $organizer['cn'] : '';
+
                     if ($action == 'reply2m') {
                         $startStamp = time();
                         $endStamp = $startStamp + (60 * 24 * 3600);
@@ -361,13 +400,6 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                     // Find out who we are and update status.
                     $identity = &Identity::singleton();
                     $email = $identity->getFromAddress();
-                    $cn = $identity->getValue('fullname');
-
-                    // Get the organizer details.
-                    $organizer = parse_url($vFb->getAttribute('ORGANIZER'));
-                    $organizerEmail = $organizer['path'];
-                    $organizer = $vFb->getAttribute('ORGANIZER', true);
-                    $organizerName = isset($organizer['cn']) ? $organizer['cn'] : '';
 
                     // Build the reply.
                     $vCal = new Horde_iCalendar();
@@ -406,7 +438,10 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                     $msg_headers->addMIMEHeaders($mime);
 
                     // Send the reply.
-                    $status = $mime->send($organizerEmail, $msg_headers);
+                    $mail_driver = $this->_getMailDriver();
+                    $status = $mime->send($organizerEmail, $msg_headers,
+                                          $mail_driver['driver'],
+                                          $mail_driver['params']);
                     if (is_a($status, 'PEAR_Error')) {
                         $this->_msgs[$key][] = array('error', sprintf(_("Error sending reply: %s."), $status->getMessage()));
                     } else {
@@ -430,7 +465,9 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
         if (MIME_Contents::viewAsAttachment()) {
             $html .= Util::bufferOutput('require', $registry->get('templates', 'horde') . '/common-header.inc');
         }
-        $html .= '<form method="post" name="iCal" action="' . Horde::selfUrl(true) . '">';
+        if ($_SESSION['imp']['viewmode'] == 'imp') {
+            $html .= '<form method="post" name="iCal" action="' . Horde::selfUrl(true) . '">';
+        }
 
         foreach ($components as $key => $component) {
             switch ($component->getType()) {
@@ -438,6 +475,10 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
                 $html .= $this->_vEvent($component, $key);
                 break;
 
+            case 'vTodo':
+                $html .= $this->_vTodo($component, $key);
+                break;
+
             case 'vTimeZone':
                 // Ignore them.
                 break;
@@ -453,7 +494,9 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
         }
 
         // Need to work out if we are inline and actually need this.
-        $html .= '</form>';
+        if ($_SESSION['imp']['viewmode'] == 'imp') {
+            $html .= '</form>';
+        }
         if (MIME_Contents::viewAsAttachment()) {
             $html .= Util::bufferOutput('require', $registry->get('templates', 'horde') . '/common-footer.inc');
         }
@@ -471,6 +514,52 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
         return 'text/html; charset=' . NLS::getCharset();
     }
 
+    /**
+     * Return mail driver/params necessary to send a message.
+     *
+     * @return array  'driver' => mail driver; 'params' => list of params.
+     */
+    function _getMailDriver()
+    {
+        global $conf;
+
+        /* We don't actually want to alter the contents of the $conf['mailer']
+         * array, so we make a copy of the current settings. We will apply our
+         * modifications (if any) to the copy, instead. */
+        $params = $conf['mailer']['params'];
+        $driver = $conf['mailer']['type'];
+
+        /* If user specifies an SMTP server on login, force SMTP mailer. */
+        if (!empty($conf['server']['change_smtphost'])) {
+            $driver = 'smtp';
+            if (empty($params['mailer']['auth'])) {
+                $params['mailer']['auth'] = '1';
+            }
+        }
+
+        /* Force the SMTP host and port value to the current SMTP server if
+         * one has been selected for this connection. */
+        if (!empty($_SESSION['imp']['smtphost'])) {
+            $params['host'] = $_SESSION['imp']['smtphost'];
+        }
+        if (!empty($_SESSION['imp']['smtpport'])) {
+            $params['port'] = $_SESSION['imp']['smtpport'];
+        }
+
+        /* If SMTP authentication has been requested, use either the username
+         * and password provided in the configuration or populate the username
+         * and password fields based on the current values for the user. Note
+         * that we assume that the username and password values from the
+         * current IMAP / POP3 connection are valid for SMTP authentication as
+         * well. */
+        if (!empty($params['auth']) && empty($params['username'])) {
+            $params['username'] = $_SESSION['imp']['user'];
+            $params['password'] = Secret::read(Secret::getKey('imp'), $_SESSION['imp']['pass']);
+        }
+
+        return array('driver' => $driver, 'params' => $params);
+    }
+
     /**
      * Return the html for a vFreebusy.
      */
@@ -509,7 +598,7 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
             if (is_array($start)) {
                 $html .= '<p><strong>' . _("Start") . ':</strong> ' . strftime($prefs->getValue('date_format'), mktime(0, 0, 0, $start['month'], $start['mday'], $start['year'])) . '</p>';
             } else {
-                $html .= '<p><strong>' . _("Start") . ':</strong> ' . strftime($prefs->getValue('date_format') . ' ' . $prefs->getValue('time_format'), $start) . '</p>';
+                $html .= '<p><strong>' . _("Start") . ':</strong> ' . strftime($prefs->getValue('date_format'), $start) . ' ' . date($prefs->getValue('twentyFour') ? ' G:i' : ' g:i a', $start) . '</p>';
             }
         }
 
@@ -518,10 +607,14 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
             if (is_array($end)) {
                 $html .= '<p><strong>' . _("End") . ':</strong> ' . strftime($prefs->getValue('date_format'), mktime(0, 0, 0, $end['month'], $end['mday'], $end['year'])) . '</p>';
             } else {
-                $html .= '<p><strong>' . _("End") . ':</strong> ' . strftime($prefs->getValue('date_format') . ' ' . $prefs->getValue('time_format'), $end) . '</p>';
+                $html .= '<p><strong>' . _("End") . ':</strong> ' . strftime($prefs->getValue('date_format'), $end) . ' ' . date($prefs->getValue('twentyFour') ? ' G:i' : ' g:i a', $end) . '</p>';
             }
         }
 
+        if ($_SESSION['imp']['viewmode'] != 'imp') {
+            return $html;
+        }
+
         $html .= '<h2 class="smallheader">' . _("Actions") . '</h2>' .
             '<select name="action[' . $id . ']">';
 
@@ -569,6 +662,14 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
         $sender = $vevent->organizerName();
         $options = array();
 
+        $attendees = $vevent->getAttribute('ATTENDEE');
+        if (!is_a($attendees, 'PEAR_Error') &&
+            !empty($attendees) &&
+            !is_array($attendees)) {
+            $attendees = array($attendees);
+        }
+        $attendee_params = $vevent->getAttribute('ATTENDEE', true);
+
         switch ($this->_method) {
         case 'PUBLISH':
             $desc = _("%s wishes to make you aware of \"%s\".");
@@ -578,11 +679,39 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
             break;
 
         case 'REQUEST':
-            // Check that you are one of the attendees here.
-            $desc = _("%s requests your presence at \"%s\".");
-            if ($registry->hasMethod('calendar/import')) {
-                $options[] = '<option value="accept-import">' .   _("Accept and add to my calendar") . '</option>';
-                $options[] = '<option value="import">' .   _("Add to my calendar") . '</option>';
+            // Check if this is an update.
+            if ($registry->hasMethod('calendar/export') &&
+                !is_a($registry->call('calendar/export', array($vevent->getAttribute('UID'), 'text/calendar')), 'PEAR_Error')) {
+                $is_update = true;
+                $desc = _("%s wants to notify you about changes of \"%s\".");
+            } else {
+                $is_update = false;
+
+                // Check that you are one of the attendees here.
+                $is_attendee = false;
+                if (!is_a($attendees, 'PEAR_Error') && !empty($attendees)) {
+                    require_once 'Horde/Identity.php';
+                    $identity = &Identity::singleton(array('imp', 'imp'));
+                    for ($i = 0, $c = count($attendees); $i < $c; ++$i) {
+                        $attendee = parse_url($attendees[$i]);
+                        if (!empty($attendee['path']) &&
+                            $identity->hasAddress($attendee['path'])) {
+                            $is_attendee = true;
+                            break;
+                        }
+                    }
+                }
+
+                $desc = $is_attendee
+                    ? _("%s requests your presence at \"%s\".")
+                    : _("%s wishes to make you aware of \"%s\".");
+            }
+            if ($is_update && $registry->hasMethod('calendar/replace')) {
+                $options[] = '<option value="accept-import">' . _("Accept and update in my calendar") . '</option>';
+                $options[] = '<option value="import">' . _("Update in my calendar") . '</option>';
+            } elseif ($registry->hasMethod('calendar/import')) {
+                $options[] = '<option value="accept-import">' . _("Accept and add to my calendar") . '</option>';
+                $options[] = '<option value="import">' . _("Add to my calendar") . '</option>';
             }
             $options[] = '<option value="accept">' . _("Accept request") . '</option>';
             $options[] = '<option value="tentative">' . _("Tentatively Accept request") . '</option>';
@@ -600,14 +729,6 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
         case 'REFRESH':
             $desc = _("%s wishes to receive the latest information about \"%s\".");
             $options[] = '<option value="send">' . _("Send Latest Information") . '</option>';
-            if (!$found && $registry->hasMethod('calendar/eventFromGUID')) {
-                $existing_vevent = $registry->call('calendar/eventFromGUID', array('guid' => $vevent->getAttribute('UID')));
-                if (!is_a($existing_vevent, 'Pear_error')) {
-                    $existing_vevent->updateFromvEvent($vevent);
-                    $vevent = $existing_vevent;
-                    $found = true;
-                }
-            }
             break;
 
         case 'REPLY':
@@ -619,9 +740,16 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
             break;
 
         case 'CANCEL':
-            $desc = _("%s has cancelled \"%s\".");
-            if ($registry->hasMethod('calendar/delete')) {
-                $options[] = '<option value="delete">' . _("Delete from my calendar") . '</option>';
+            if (is_a($instance = $vevent->getAttribute('RECURRENCE-ID'), 'PEAR_Error')) {
+                $desc = _("%s has cancelled \"%s\".");
+                if ($registry->hasMethod('calendar/delete')) {
+                    $options[] = '<option value="delete">' . _("Delete from my calendar") . '</option>';
+                }
+            } else {
+                $desc = _("%s has cancelled an instance of the recurring \"%s\".");
+                if ($registry->hasMethod('calendar/replace')) {
+                    $options[] = '<option value="import">' . _("Update in my calendar") . '</option>';
+                }
             }
             break;
         }
@@ -646,7 +774,7 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
             if (is_array($start)) {
                 $html .= '<p><strong>' . _("Start") . ':</strong> ' . strftime($prefs->getValue('date_format'), mktime(0, 0, 0, $start['month'], $start['mday'], $start['year'])) . '</p>';
             } else {
-                $html .= '<p><strong>' . _("Start") . ':</strong> ' . strftime($prefs->getValue('date_format') . ' ' . $prefs->getValue('time_format'), $start) . '</p>';
+                $html .= '<p><strong>' . _("Start") . ':</strong> ' . strftime($prefs->getValue('date_format'), $start) . ' ' . date($prefs->getValue('twentyFour') ? ' G:i' : ' g:i a', $start) . '</p>';
             }
         }
 
@@ -655,7 +783,7 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
             if (is_array($end)) {
                 $html .= '<p><strong>' . _("End") . ':</strong> ' . strftime($prefs->getValue('date_format'), mktime(0, 0, 0, $end['month'], $end['mday'], $end['year'])) . '</p>';
             } else {
-                $html .= '<p><strong>' . _("End") . ':</strong> ' . strftime("%x %X", $end) . '</p>';
+                $html .= '<p><strong>' . _("End") . ':</strong> ' . strftime($prefs->getValue('date_format'), $end) . ' ' . date($prefs->getValue('twentyFour') ? ' G:i' : ' g:i a', $end) . '</p>';
             }
         }
 
@@ -673,11 +801,137 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
 
         $loc = $vevent->getAttribute('LOCATION');
         if (!is_a($loc, 'PEAR_Error')) {
-            $html .= '<p><strong>' . _("Location") . ':</strong>' . htmlspecialchars($loc) . '</p>';
+            $html .= '<p><strong>' . _("Location") . ':</strong> ' . htmlspecialchars($loc) . '</p>';
         }
 
-        $attendees = $vevent->getAttribute('ATTENDEE');
-        $params = $vevent->getAttribute('ATTENDEE', true);
+        if (!is_a($attendees, 'PEAR_Error') && !empty($attendees)) {
+            $html .= '<h2 class="smallheader">' . _("Attendees") . '</h2>';
+
+            $html .= '<table><thead class="leftAlign"><tr><th>' . _("Name") . '</th><th>' . _("Role") . '</th><th>' . _("Status") . '</th></tr></thead><tbody>';
+            foreach ($attendees as $key => $attendee) {
+                $attendee = parse_url($attendee);
+                $attendee = empty($attendee['path']) ? _("Unknown") : $attendee['path'];
+
+                if (!empty($attendee_params[$key]['CN'])) {
+                    $attendee = $attendee_params[$key]['CN'];
+                }
+
+                $role = _("Required Participant");
+                if (isset($attendee_params[$key]['ROLE'])) {
+                    switch ($attendee_params[$key]['ROLE']) {
+                    case 'CHAIR':
+                        $role = _("Chair Person");
+                        break;
+
+                    case 'OPT-PARTICIPANT':
+                        $role = _("Optional Participant");
+                        break;
+
+                    case 'NON-PARTICIPANT':
+                        $role = _("Non Participant");
+                        break;
+
+                    case 'REQ-PARTICIPANT':
+                    default:
+                        // Already set above.
+                        break;
+                    }
+                }
+
+                $status = _("Awaiting Response");
+                if (isset($attendee_params[$key]['PARTSTAT'])) {
+                    $status = $this->_partstatToString($attendee_params[$key]['PARTSTAT'], $status);
+                }
+
+                $html .= '<tr><td>' . htmlspecialchars($attendee) . '</td><td>' . htmlspecialchars($role) . '</td><td>' . htmlspecialchars($status) . '</td></tr>';
+            }
+            $html .= '</tbody></table>';
+        }
+
+        if ($_SESSION['imp']['viewmode'] != 'imp') {
+            return $html;
+        }
+
+        if ($options) {
+            $html .= '<h2 class="smallheader">' . _("Actions") . '</h2>' .
+                '<label for="action_' . $id . '" class="hidden">' . _("Actions") . '</label>' .
+                '<select id="action_' . $id . '" name="action[' . $id . ']">' .
+                implode("\n", $options) .
+                '</select> <input type="submit" class="button" value="' . _("Go") . '" />';
+        }
+
+        return $html;
+    }
+
+    /**
+     * Returns the html for a vEvent.
+     *
+     * @todo IMP 5: move organizerName() from Horde_iCalendar_vevent to
+     *       Horde_iCalendar
+     */
+    function _vTodo($vtodo, $id)
+    {
+        global $registry, $prefs;
+
+        $html = '';
+        $desc = '';
+        $options = array();
+
+        $organizer = $vtodo->getAttribute('ORGANIZER', true);
+        if (is_a($organizer, 'PEAR_Error')) {
+            $sender = _("An unknown person");
+        } else {
+            if (isset($organizer[0]['CN'])) {
+                $sender = $organizer[0]['CN'];
+            } else {
+                $organizer = parse_url($vtodo->getAttribute('ORGANIZER'));
+                $sender = $organizer['path'];
+            }
+        }
+
+        switch ($this->_method) {
+        case 'PUBLISH':
+            $desc = _("%s wishes to make you aware of \"%s\".");
+            if ($registry->hasMethod('tasks/import')) {
+                $options[] = '<option value="import">' . _("Add this to my tasklist") . '</option>';
+            }
+            break;
+        }
+
+        $summary = $vtodo->getAttribute('SUMMARY');
+        if (is_a($summary, 'PEAR_Error')) {
+            $desc = sprintf($desc, htmlspecialchars($sender), _("Unknown Task"));
+        } else {
+            $desc = sprintf($desc, htmlspecialchars($sender), htmlspecialchars($summary));
+        }
+
+        $html .= '<h2 class="header">' . $desc . '</h2>';
+
+        if ($this->_msgs) {
+            foreach ($this->_msgs[$id] as $msg) {
+                $html .= '<p class="notice">' . Horde::img('alerts/' . $msg[0] . '.png', '', null, $registry->getImageDir('horde')) . $msg[1] . '</p>';
+            }
+        }
+
+        $priority = $vtodo->getAttribute('PRIORITY');
+        if (!is_a($priority, 'PEAR_Error')) {
+            $html .= '<p><strong>' . _("Priority") . ':</strong> ' . (int)$priority . '</p>';
+        }
+
+        $sum = $vtodo->getAttribute('SUMMARY');
+        if (!is_a($sum, 'PEAR_Error')) {
+            $html .= '<p><strong>' . _("Summary") . ':</strong> ' . htmlspecialchars($sum) . '</p>';
+        } else {
+            $html .= '<p><strong>' . _("Summary") . ':</strong> <em>' . _("None") . '</em></p>';
+        }
+
+        $desc = $vtodo->getAttribute('DESCRIPTION');
+        if (!is_a($desc, 'PEAR_Error')) {
+            $html .= '<p><strong>' . _("Description") . ':</strong> ' . nl2br(htmlspecialchars($desc)) . '</p>';
+        }
+
+        $attendees = $vtodo->getAttribute('ATTENDEE');
+        $params = $vtodo->getAttribute('ATTENDEE', true);
 
         if (!is_a($attendees, 'PEAR_Error') && !empty($attendees)) {
             $html .= '<h2 class="smallheader">' . _("Attendees") . '</h2>';
@@ -688,7 +942,7 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
             $html .= '<table><thead class="leftAlign"><tr><th>' . _("Name") . '</th><th>' . _("Role") . '</th><th>' . _("Status") . '</th></tr></thead><tbody>';
             foreach ($attendees as $key => $attendee) {
                 $attendee = parse_url($attendee);
-                $attendee = empty($attendee['path']) ? _("Unknown") : $attendee['path'];
+                $attendee = $attendee['path'];
 
                 if (isset($params[$key]['CN'])) {
                     $attendee = $params[$key]['CN'];
@@ -726,6 +980,10 @@ class IMP_MIME_Viewer_itip extends MIME_Viewer {
             $html .= '</tbody></table>';
         }
 
+        if ($_SESSION['imp']['viewmode'] != 'imp') {
+            return $html;
+        }
+
         if ($options) {
             $html .= '<h2 class="smallheader">' . _("Actions") . '</h2>' .
                 '<select name="action[' . $id . ']">' .
