@@ -4,8 +4,7 @@ require_once IMP_BASE . '/lib/Crypt/PGP.php';
 
 /**
  * The IMP_MIME_Viewer_pgp class allows viewing/decrypting of PGP
- * formatted messages.  This class implements RFC 2015 and is also
- * compliant with the proposed revision, RFC 3156.
+ * formatted messages.  This class implements RFC 3156.
  *
  * This class handles the following MIME types:
  *   application/pgp-encryption
@@ -13,17 +12,17 @@ require_once IMP_BASE . '/lib/Crypt/PGP.php';
  *   application/pgp-signature
  *
  * This class may add the following parameters to the URL:
+ *   'pgp_verify_msg' -- Do verification of PGP signed data.
  *   'rawpgpkey' -- Display the PGP Public Key in raw, text format
  *
- * $Horde: imp/lib/MIME/Viewer/pgp.php,v 1.96.6.6 2005/09/30 03:22:37 selsky Exp $
+ * $Horde: imp/lib/MIME/Viewer/pgp.php,v 1.96.6.24 2009/01/06 15:24:09 jan Exp $
  *
- * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @since   IMP 4.0
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @package Horde_MIME_Viewer
  */
 class IMP_MIME_Viewer_pgp extends MIME_Viewer {
@@ -52,7 +51,7 @@ class IMP_MIME_Viewer_pgp extends MIME_Viewer {
     /**
      * Classwide cache for icons for status messages.
      *
-     * @var string 
+     * @var string
      */
     var $_icon = null;
 
@@ -88,7 +87,7 @@ class IMP_MIME_Viewer_pgp extends MIME_Viewer {
         $msg = '';
 
         if (empty($this->_imp_pgp) && !empty($conf['utils']['gnupg'])) {
-            $this->_imp_pgp = &new IMP_PGP();
+            $this->_imp_pgp = new IMP_PGP();
         }
 
         /* Determine the address of the sender. */
@@ -101,7 +100,8 @@ class IMP_MIME_Viewer_pgp extends MIME_Viewer {
         if (!empty($conf['utils']['gnupg']) &&
             $prefs->getValue('use_pgp') &&
             !Util::getFormData('rawpgpkey')) {
-            $msg = Util::bufferOutput('require_once', IMP_TEMPLATES . '/pgp/open_pgp_win.js');
+            $msg = Util::bufferOutput(array('Horde', 'addScriptFile'), 'prototype.js', 'imp', true);
+            $msg .= Util::bufferOutput(array('Horde', 'addScriptFile'), 'popup.js', 'imp', true);
         }
 
         /* For RFC 2015/3156, there are 3 types of messages:
@@ -163,15 +163,14 @@ class IMP_MIME_Viewer_pgp extends MIME_Viewer {
 
             /* Initialize status message. */
             $this->_initStatus($this->getIcon($mime->getType()), _("PGP"));
-            $this->_status[] = _("This PGP Public Key was attached to the message.");
-
+            $msg = _("This PGP Public Key was attached to the message.");
             if ($prefs->getValue('use_pgp') &&
                 $GLOBALS['registry']->hasMethod('contacts/addField') &&
                 $prefs->getValue('add_source')) {
-                $this->_status[] = Horde::link('', _("Click to Save PGP Public Key in your Address book"), null, null, $this->_imp_pgp->savePublicKeyURL($mime) . ' return false;') . _("Click to Save PGP Public Key in your Address book") . '</a>.';
+                $msg .= ' ' . Horde::link('#', '', '', '', $this->_imp_pgp->savePublicKeyURL($mime) . ' return false;') . _("[Save the key to your Address book]") . '</a>';
             }
-            $this->_status[] = sprintf(_("The raw text of the PGP Public Key can be viewed %s."), $this->_contents->linkViewJS($part, 'view_attach', _("HERE"), _("PGP Public Key Text"), null, array('rawpgpkey' => 1)));
-            
+            $this->_status[] = $msg . ' ' . $this->_contents->linkViewJS($part, 'view_attach', _("[View the raw key]"), '', null, array('rawpgpkey' => 1));
+
             $text = $this->_outputStatus(false) .
                 '<span class="fixed">' . nl2br(str_replace(' ', '&nbsp;', $this->_imp_pgp->pgpPrettyKey($pgp_key))) . '</span>';
         }
@@ -197,19 +196,24 @@ class IMP_MIME_Viewer_pgp extends MIME_Viewer {
         $text = '';
 
         $signenc = $mime->getInformation('pgp_signenc');
-        if ($signenc) {
-            $this->_status[] = _("This message has been encrypted with PGP.");
+        if (!$active) {
+            if ($signenc) {
+                $this->_status[] = _("The message below has been digitally signed and encrypted with PGP, but the signature cannot be verified.");
+            } else {
+                $this->_status[] = _("The message below has been digitally signed with PGP, but the signature cannot be verified.");
+            }
+        } else {
+            if ($signenc) {
+                $this->_status[] = _("The message below has been digitally signed and encrypted with PGP.");
+            } else {
+                $this->_status[] = _("The message below has been digitally signed with PGP.");
+            }
         }
 
         $this->_initStatus($this->getIcon($mimetype), _("PGP"));
-        $this->_status[] = _("This message has been digitally signed via PGP.");
-
-        if (!$active) {
-            $this->_status[] = _("PGP support is not currently active so the digital signature is unable to be verified.");
-        }
 
         /* Store PGP results in $sig_result; store text in $data. */
-        $sig_result = '';
+        $sig_result = null;
         if ($mimetype == 'multipart/signed') {
             /* If the MIME ID is 0, we need to store the body of the message
                in the MIME_Part object. */
@@ -233,16 +237,24 @@ class IMP_MIME_Viewer_pgp extends MIME_Viewer {
             $subpart = $mime->getPart($mime->getRelativeMIMEId(2));
             if ($subpart && $subpart->getType() == 'application/pgp-signature') {
                 if ($active) {
-                    $subpart->transferDecodeContents();
-                    $sig_result = $this->_imp_pgp->verifySignature($signature_data, $this->_address, $subpart->getContents());
+                    if ($GLOBALS['prefs']->getValue('pgp_verify') ||
+                        Util::getFormData('pgp_verify_msg')) {
+                        $subpart->transferDecodeContents();
+                        $sig_result = $this->_imp_pgp->verifySignature($signature_data, $this->_address, $subpart->getContents());
+                    } elseif (isset($_SESSION['imp']['viewmode']) &&
+                              ($_SESSION['imp']['viewmode'] == 'imp')) {
+                        // TODO: Fix to work with DIMP
+                        $base_ob = &$this->_contents->getBaseObjectPtr();
+                        $this->_status[] = Horde::link(Util::addParameter(IMP::generateIMPUrl(Horde::selfUrl(), $GLOBALS['imp_mbox']['mailbox'], $GLOBALS['imp_mbox']['index'], $GLOBALS['imp_mbox']['thismailbox']), 'pgp_verify_msg', 1)) . _("Click HERE to verify the message.") . '</a>';
+                    }
                 }
             } else {
-                $this->_status[] = _("This message does not appear to be in the correct PGP format (according to RFC 2015).");
+                $this->_status[] = _("The message below does not appear to be in the correct PGP format (according to RFC 2015).");
             }
         } elseif ($mimetype == 'application/pgp-signature') {
             /* Get the signed message to output. */
             require_once 'Horde/MIME/Message.php';
-            $mime_message = &new MIME_Message();
+            $mime_message = new MIME_Message();
             $mime_message->setType('text/plain');
             $mime->transferDecodeContents();
 
@@ -254,21 +266,28 @@ class IMP_MIME_Viewer_pgp extends MIME_Viewer {
 
             /* Pass the signed text straight through to PGP program */
             if ($active) {
-                $sig_result = $this->_imp_pgp->verifySignature($mime->getContents(), $this->_address);
+                if ($GLOBALS['prefs']->getValue('pgp_verify') ||
+                    Util::getFormData('pgp_verify_msg')) {
+                    $sig_result = $this->_imp_pgp->verifySignature($mime->getContents(), $this->_address);
+                } elseif (isset($_SESSION['imp']['viewmode']) &&
+                          ($_SESSION['imp']['viewmode'] == 'imp')) {
+                    // TODO: Fix to work with DIMP
+                    $this->_status[] = Horde::link(Util::addParameter(Horde::selfUrl(true), 'pgp_verify_msg', 1)) . _("Click HERE to verify the message.") . '</a>';
+                }
             }
         }
 
         $text = $this->_outputStatus();
 
-        if ($active) {
+        if ($sig_result !== null) {
             $text .= $this->_outputPGPSignatureTest($sig_result);
         }
 
         /* We need to stick the output into a MIME_Contents object. */
-        $mc = &new MIME_Contents($mime_message, array('download' => 'download_attach', 'view' => 'view_attach'), array(&$this->_contents));
+        $mc = new MIME_Contents($mime_message, array('download' => 'download_attach', 'view' => 'view_attach'), array(&$this->_contents));
         $mc->buildMessage();
 
-        return $text . '<table cellpadding="0" cellspacing="0">' . $mc->getMessage(true) . '</table>';
+        return $text . '<table cellspacing="0">' . $mc->getMessage(true) . '</table>';
     }
 
     /**
@@ -288,31 +307,73 @@ class IMP_MIME_Viewer_pgp extends MIME_Viewer {
         $text = '';
 
         $this->_initStatus($this->getIcon($mimetype), _("PGP"));
-        $this->_status[] = _("This message has been encrypted with PGP.");
 
         /* Print out message now if PGP is not active. */
         if (empty($conf['utils']['gnupg']) || !$prefs->getValue('use_pgp')) {
-            $this->_status[] = _("PGP support is not currently active so the message is unable to be decrypted.");
+            $this->_status[] = _("The message below has been encrypted with PGP, however, PGP support is disabled so the message cannot be decrypted.");
             return $this->_outputStatus();
         }
 
-        $passphrase = $this->_imp_pgp->getPassphrase();
-        if (empty($passphrase)) {
-            $url = $this->_imp_pgp->getJSOpenWinCode('open_passphrase_dialog');
-            $this->_status[] = Horde::link('', _("You must enter the passphrase for your PGP private key to view this message"), null, null, $url . ' return false;') . _("You must enter the passphrase for your PGP private key to view this message") . '</a>';
-            $text .= $this->_outputStatus() .
-                '<script language="JavaScript" type="text/javascript">' . $url . '</script>';
+        if ($mimetype == 'multipart/encrypted') {
+            /* PGP control information appears in the first MIME subpart. We
+               don't currently need to do anything with this information. The
+               encrypted data appears in the second MIME subpart. */
+            $subpart = $mime->getPart($mime->getRelativeMIMEId(2));
+            if (!$subpart) {
+                return $text;
+            }
+            $encrypted_data = $this->_contents->getBodyPart($subpart->getMIMEId());
+        } elseif ($mimetype == 'application/pgp-encrypted') {
+            $encrypted_data = $mime->getContents();
         } else {
+            return $text;
+        }
+
+        /* Check if this is a literal compressed message. */
+        $info = $this->_imp_pgp->pgpPacketInformation($encrypted_data);
+        $literal = !empty($info['literal']);
+
+        /* Check if this a symmetrically encrypted message. */
+        $symmetric = is_callable(array($this->_imp_pgp, 'encryptedSymmetrically'))
+            ? $this->_imp_pgp->encryptedSymmetrically($encrypted_data)
+            : false;
+
+        if ($symmetric && !$this->_imp_pgp->getSymmetricPassphrase()) {
+            if (isset($_SESSION['imp']['viewmode']) &&
+                ($_SESSION['imp']['viewmode'] == 'imp')) {
+                // TODO: Fix to work with DIMP
+                /* Ask for the correct passphrase if this is encrypted
+                 * symmetrically. */
+                $url = $this->_imp_pgp->getJSOpenWinCode('open_symmetric_passphrase_dialog');
+                $this->_status[] = Horde::link('#', _("The message below has been encrypted with PGP. You must enter the passphrase that was used to encrypt this message."), null, null, $url . ' return false;') . _("You must enter the passphrase that was used to encrypt this message.") . '</a>';
+                $text .= $this->_outputStatus() .
+                    Util::bufferOutput(array('IMP', 'addInlineScript'), $url);
+            }
+        } elseif (!$literal && !$symmetric &&
+                  !$this->_imp_pgp->getPersonalPrivateKey()) {
+            /* Output if there is no personal private key to decrypt with. */
+            $this->_status[] = _("The message below has been encrypted with PGP, however, no personal private key exists so the message cannot be decrypted.");
+            return $this->_outputStatus();
+        } elseif (!$literal && !$symmetric &&
+                  !$this->_imp_pgp->getPassphrase()) {
+            if (isset($_SESSION['imp']['viewmode']) &&
+                ($_SESSION['imp']['viewmode'] == 'imp')) {
+                // TODO: Fix to work with DIMP
+                /* Ask for the private key's passphrase if this is encrypted
+                 * asymmetrically. */
+                $url = $this->_imp_pgp->getJSOpenWinCode('open_passphrase_dialog');
+                $this->_status[] = Horde::link('#', _("The message below has been encrypted with PGP. You must enter the passphrase for your PGP private key before it can be decrypted."), null, null, $url . ' return false;') . _("You must enter the passphrase for your PGP private key to view this message.") . '</a>';
+                $text .= $this->_outputStatus() .
+                    Util::bufferOutput(array('IMP', 'addInlineScript'), $url);
+            }
+        } else {
+            /* Decrypt this message. */
+            $this->_status[] = $literal ? _("The message below has been compressed with PGP.") : _("The message below has been encrypted with PGP.");
             if ($mimetype == 'multipart/encrypted') {
-                /* PGP control information appears in the first MIME
-                   subpart. We don't currently need to do anything with
-                   this information. The encrypted data appears in the
-                   second MIME subpart. */
-                $subpart = $mime->getPart($mime->getRelativeMIMEId(2));
                 if ($subpart->getType() == 'application/octet-stream') {
-                    $decrypted_data = $this->_imp_pgp->decryptMessage($this->_contents->getBodyPart($subpart->getMIMEId()), $this->_address);
+                    $decrypted_data = $this->_imp_pgp->decryptMessage($encrypted_data, $symmetric, !$literal);
                     if (is_a($decrypted_data, 'PEAR_Error')) {
-                        $this->_status[] = _("This message does not appear to be a valid PGP encrypted message. Error: ") . $decrypted_data->getMessage();
+                        $this->_status[] = _("The message below does not appear to be a valid PGP encrypted message. Error: ") . $decrypted_data->getMessage();
                         $text .= $this->_outputStatus();
                     } else {
                         /* We need to check if this is a signed/encrypted
@@ -320,7 +381,7 @@ class IMP_MIME_Viewer_pgp extends MIME_Viewer {
                         require_once 'Horde/MIME/Structure.php';
                         $mime_message = &MIME_Structure::parseTextMIMEMessage($decrypted_data->message);
                         if (!$mime_message) {
-                            require_once 'Horde/Text/Filter.php'; 
+                            require_once 'Horde/Text/Filter.php';
                             $text .= $this->_signedOutput($decrypted_data->sig_result);
                             $decrypted_message = String::convertCharset($decrypted_data->message, $subpart->getCharset());
                             $text .= '<span class="fixed">' . Text_Filter::filter($decrypted_message, 'text2html', array('parselevel' => TEXT_HTML_SYNTAX)) . '</span>';
@@ -337,21 +398,21 @@ class IMP_MIME_Viewer_pgp extends MIME_Viewer {
 
                             /* We need to stick the output into a
                                MIME_Contents object. */
-                            $mc = &new MIME_Contents($mime_message, array('download' => 'download_attach', 'view' => 'view_attach'), array(&$this->_contents));
+                            $mc = new MIME_Contents($mime_message, array('download' => 'download_attach', 'view' => 'view_attach'), array(&$this->_contents));
                             $mc->buildMessage();
                             if ($mime_message->getInformation('pgp_signenc')) {
                                 $text .= $mc->getMessage(true);
                             } else {
-                                $text .= '<table cellpadding="0" cellspacing="0">' . $mc->getMessage(true) . '</table>';
+                                $text .= '<table cellspacing="0">' . $mc->getMessage(true) . '</table>';
                             }
                         }
                     }
                 } else {
-                    $this->_status[] = _("This message does not appear to be in the correct PGP format (according to RFC 2015).");
+                    $this->_status[] = _("The message below does not appear to be in the correct PGP format (according to RFC 2015).");
                     $text .= $this->_outputStatus();
                 }
             } elseif ($mimetype == 'application/pgp-encrypted') {
-                $decrypted_data = $this->_imp_pgp->decryptMessage($mime->getContents(), $this->_address);
+                $decrypted_data = $this->_imp_pgp->decryptMessage($encrypted_data, $symmetric, !$literal);
                 if (is_a($decrypted_data, 'PEAR_Error')) {
                     $decrypted_message = $decrypted_data->getMessage();
                     $text .= $this->_outputStatus();
@@ -360,10 +421,11 @@ class IMP_MIME_Viewer_pgp extends MIME_Viewer {
                     $decrypted_message = String::convertCharset($decrypted_data->message, $mime->getCharset());
                 }
 
-                require_once 'Horde/Text/Filter.php'; 
+                require_once 'Horde/Text/Filter.php';
                 $text .= '<span class="fixed">' . Text_Filter::filter($decrypted_message, 'text2html', array('parselevel' => TEXT_HTML_SYNTAX)) . '</span>';
             }
         }
+        $this->_imp_pgp->unsetSymmetricPassphrase();
 
         return $text;
     }
@@ -387,15 +449,14 @@ class IMP_MIME_Viewer_pgp extends MIME_Viewer {
             $result = $result->getMessage();
         } else {
             $this->_initStatus($GLOBALS['registry']->getImageDir('horde') . '/alerts/success.png', _("Success"));
-            /* This message has been verified but there was no output
-               from the PGP program. */
+            /* This message has been verified but there was no output from the
+               PGP program. */
             if (empty($result)) {
-               $result = _("The message has been verified.");
+               $result = _("The message below has been verified.");
             }
         }
 
         require_once 'Horde/Text/Filter.php';
-
         $this->_status[] = Text_Filter::filter($result, 'text2html', array('parselevel' => TEXT_HTML_NOHTML));
 
         return $this->_outputStatus();
@@ -413,21 +474,17 @@ class IMP_MIME_Viewer_pgp extends MIME_Viewer {
     function _signedOutput($result)
     {
         if (!empty($result)) {
-            $this->_status[] = _("This message has been digitally signed via PGP.");
-            $text = $this->_outputStatus() .
-                $this->_outputPGPSignatureTest($result);
+            return $this->_outputPGPSignatureTest($result);
         } else {
-            $text = $this->_outputStatus();
+            return $this->_outputStatus();
         }
-
-        return $text;
     }
 
     /* Various formatting helper functions */
     function _initStatus($src, $alt = '')
     {
-        if (is_null($this->_icon)) {
-            $this->_icon = Horde::img($src, $alt, 'height="16" width="16"', '') . '&nbsp;';
+        if ($this->_icon === null) {
+            $this->_icon = Horde::img($src, $alt, 'height="16" width="16"', '');
         }
     }
 
